"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createControllerContext = void 0;
var jsx_runtime_1 = require("react/jsx-runtime");
var react_1 = require("react");
var useController_1 = require("./useController");
function createControllerContext() {
    var reactContext = (0, react_1.createContext)(undefined);
    var controllerContext = {
        Provider: null,
        Builder: null,
        Listener: null,
        _context: reactContext,
    };
    var ControllerProvider = function (_a) {
        var create = _a.create, children = _a.children;
        var controllerRef = (0, react_1.useRef)(create());
        (0, react_1.useEffect)(function () { return function () {
            controllerRef.current.dispose();
        }; }, []);
        return (0, jsx_runtime_1.jsx)(reactContext.Provider, { value: controllerRef.current, children: children });
    };
    controllerContext.Provider = ControllerProvider;
    var stateReducer = function (state, action) {
        var _a;
        switch (action.type) {
            case 'newstate':
                if (state === action.payload) {
                    return state;
                }
                if (action.buildWhen) {
                    if ((_a = action.buildWhen) === null || _a === void 0 ? void 0 : _a.call(action, state, action.payload)) {
                        return action.payload;
                    }
                    return state;
                }
                return action.payload;
        }
    };
    var Builder = function (_a) {
        var builder = _a.builder, buildWhen = _a.buildWhen;
        var controller = (0, useController_1.useController)(controllerContext);
        var _b = (0, react_1.useReducer)(stateReducer, controller.state), state = _b[0], dispatch = _b[1];
        var subcriptionRef = (0, react_1.useRef)(null);
        (0, react_1.useEffect)(function () {
            var subcription = controller.subject.subscribe(function (newState) {
                dispatch({
                    type: 'newstate',
                    payload: newState,
                    buildWhen: buildWhen,
                });
            });
            subcriptionRef.current = subcription;
            return function () {
                var _a;
                (_a = subcriptionRef.current) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            };
            // eslint-disable-next-line react-hooks/exhaustive-deps
        }, [buildWhen]);
        var renderComponent = function () {
            return builder(state);
        };
        return renderComponent();
    };
    controllerContext.Builder = Builder;
    var Listener = function (_a) {
        var listener = _a.listener, listenWhen = _a.listenWhen, children = _a.children;
        var controller = (0, useController_1.useController)(controllerContext);
        var subcriptionRef = (0, react_1.useRef)(null);
        var stateRef = (0, react_1.useRef)(controller.state);
        var emitState = function (newState) {
            listener(newState);
        };
        (0, react_1.useEffect)(function () {
            var subcription = controller.subject.subscribe(function (newState) {
                var state = stateRef.current;
                if (listenWhen) {
                    if (listenWhen === null || listenWhen === void 0 ? void 0 : listenWhen(state, newState)) {
                        emitState(newState);
                    }
                }
                else {
                    emitState(newState);
                }
                stateRef.current = newState;
            });
            subcriptionRef.current = subcription;
            return function () {
                var _a;
                (_a = subcriptionRef.current) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            };
            // eslint-disable-next-line react-hooks/exhaustive-deps
        }, [listenWhen, listener]);
        return (0, jsx_runtime_1.jsx)(jsx_runtime_1.Fragment, { children: children });
    };
    controllerContext.Listener = Listener;
    return controllerContext;
}
exports.createControllerContext = createControllerContext;
//# sourceMappingURL=ControllerContext.js.map