import { jsx as _jsx, Fragment as _Fragment } from "react/jsx-runtime";
import { createContext, useEffect, useReducer, useRef } from 'react';
import { useController } from './useController';
function createControllerContext() {
    var reactContext = createContext(undefined);
    var controllerContext = {
        Provider: null,
        Builder: null,
        Listener: null,
        _context: reactContext,
    };
    var ControllerProvider = function (_a) {
        var create = _a.create, children = _a.children;
        var controllerRef = useRef(create());
        useEffect(function () { return function () {
            controllerRef.current.dispose();
        }; }, []);
        return _jsx(reactContext.Provider, { value: controllerRef.current, children: children });
    };
    controllerContext.Provider = ControllerProvider;
    var stateReducer = function (state, action) {
        var _a;
        switch (action.type) {
            case 'newstate':
                if (state === action.payload) {
                    return state;
                }
                if (action.buildWhen) {
                    if ((_a = action.buildWhen) === null || _a === void 0 ? void 0 : _a.call(action, state, action.payload)) {
                        return action.payload;
                    }
                    return state;
                }
                return action.payload;
        }
    };
    var Builder = function (_a) {
        var builder = _a.builder, buildWhen = _a.buildWhen;
        var controller = useController(controllerContext);
        var _b = useReducer(stateReducer, controller.state), state = _b[0], dispatch = _b[1];
        var subcriptionRef = useRef(null);
        useEffect(function () {
            var subcription = controller.subject.subscribe(function (newState) {
                dispatch({
                    type: 'newstate',
                    payload: newState,
                    buildWhen: buildWhen,
                });
            });
            subcriptionRef.current = subcription;
            return function () {
                var _a;
                (_a = subcriptionRef.current) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            };
            // eslint-disable-next-line react-hooks/exhaustive-deps
        }, [buildWhen]);
        var renderComponent = function () {
            return builder(state);
        };
        return renderComponent();
    };
    controllerContext.Builder = Builder;
    var Listener = function (_a) {
        var listener = _a.listener, listenWhen = _a.listenWhen, children = _a.children;
        var controller = useController(controllerContext);
        var subcriptionRef = useRef(null);
        var stateRef = useRef(controller.state);
        var emitState = function (newState) {
            listener(newState);
        };
        useEffect(function () {
            var subcription = controller.subject.subscribe(function (newState) {
                var state = stateRef.current;
                if (listenWhen) {
                    if (listenWhen === null || listenWhen === void 0 ? void 0 : listenWhen(state, newState)) {
                        emitState(newState);
                    }
                }
                else {
                    emitState(newState);
                }
                stateRef.current = newState;
            });
            subcriptionRef.current = subcription;
            return function () {
                var _a;
                (_a = subcriptionRef.current) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            };
            // eslint-disable-next-line react-hooks/exhaustive-deps
        }, [listenWhen, listener]);
        return _jsx(_Fragment, { children: children });
    };
    controllerContext.Listener = Listener;
    return controllerContext;
}
export { createControllerContext };
//# sourceMappingURL=ControllerContext.js.map